@@ -1,7 +1,6 @@
 // ! CircuitNodeInit and various helpers for nicely implementing it
 use std::{collections::BTreeMap, iter::zip};
 
-use num_bigint::BigUint;
 use rr_util::{
     symbolic_size::{SymbolicSizeConstraint, SymbolicSizeConstraints},
     tensor_util::{Shape, TorchDeviceDtypeOp},
@@ -71,14 +70,6 @@ pub trait CircuitNodeComputeInfoImpl: CircuitNode {
         )
     }
 
-    fn compute_max_non_leaf_size(&self) -> BigUint {
-        self.children()
-            .map(|x| x.info().max_non_leaf_size.clone())
-            .chain(std::iter::once(self.info().numel()))
-            .max()
-            .unwrap()
-    }
-
     fn compute_named_axes(&self) -> NamedAxes {
         if let Some(out) = self.already_set_named_axes() {
             return out;
@@ -109,7 +100,6 @@ impl<T: CircuitNodeComputeInfoImpl + CircuitNodePrivate> CircuitNodeSetNonHashIn
         self.info_mut().is_constant = self.compute_is_constant();
         self.info_mut().is_explicitly_computable = self.compute_is_explicitly_computable();
         self.info_mut().can_be_sampled = self.compute_can_be_sampled();
-        self.info_mut().max_non_leaf_size = self.compute_max_non_leaf_size();
         self.info_mut().device_dtype = self.compute_device_dtype()?;
         self.info_mut().symbolic_size_constraints = self.compute_symbolic_size_constraints()?;
         self.info_mut().named_axes = self.compute_named_axes();
