@@ -4,7 +4,7 @@ use num_bigint::BigUint;
 use pyo3::prelude::*;
 use rr_util::{
     opt_einsum::EinsumSpec,
-    tensor_util::{TensorAxisIndex, TensorIndex, TorchDeviceDtype},
+    tensor_util::{TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDtype},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
@@ -61,7 +61,7 @@ pub fn cast_circuit(
                                                     TorchDeviceDtype::from_tensor(&tensor).device,
                                                 )
                                                 .clone(),
-                                            dtype: "int64".to_owned(),
+                                            dtype: TorchDtype::int64,
                                         }
                                         .cast_tensor((**tensor).clone())
                                         .try_into()
