@@ -12,6 +12,7 @@ use pyo3::{
 use rand::Rng;
 use regex::Regex;
 use rr_util::{
+    compact_data::TinyBoundedBitvec,
     py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS},
     python_error_exception, simple_from,
     tensor_util::{check_canon_idxs, Shape, TorchDeviceDtype},
@@ -31,7 +32,7 @@ use crate::{
 
 macro_rules! gf_gen {
     ($(($name:ident, $($t:tt)*)),* $(,)?) => {
-        pub const BASIC_SPEC_ITEMS: &'static [(&'static str, usize, usize)] = &[
+        pub const BASIC_SPEC_ITEMS: &'static [(&'static str, u8, u8)] = &[
             $(
                 gf_gen!(@item $name, $($t)*),
             )*
@@ -121,9 +122,9 @@ pub struct GeneralFunctionShapeInfo {
     #[pyo3(set)]
     pub shape: Shape,
     #[pyo3(get, set)]
-    pub num_non_batchable_output_dims: usize,
+    pub num_non_batchable_output_dims: u8,
     #[pyo3(get, set)]
-    pub input_batchability: Vec<bool>,
+    pub input_batchability: TinyBoundedBitvec,
 }
 
 #[pymethods]
@@ -137,8 +138,8 @@ impl GeneralFunctionShapeInfo {
     #[new]
     pub fn new(
         shape: Shape,
-        num_non_batchable_output_dims: usize,
-        input_batchability: Vec<bool>,
+        num_non_batchable_output_dims: u8,
+        input_batchability: TinyBoundedBitvec,
     ) -> Self {
         Self {
             shape,
@@ -154,9 +155,9 @@ pub struct GeneralFunctionSimpleSpec {
     #[pyo3(get)]
     name: String,
     #[pyo3(get)]
-    num_non_batchable_output_dims: usize,
+    num_non_batchable_output_dims: u8,
     #[pyo3(get)]
-    removed_from_end: usize,
+    removed_from_end: u8,
 }
 
 simple_from!(|x: GeneralFunctionSimpleSpec| -> GeneralFunctionSpec {
@@ -172,8 +173,8 @@ impl ToPyObject for GeneralFunctionSimpleSpec {
 #[pyfunction]
 pub fn get_shape_info_simple(
     shapes: Vec<Shape>,
-    num_non_batchable_output_dims: Option<usize>,
-    removed_from_end: Option<usize>,
+    num_non_batchable_output_dims: Option<u8>,
+    removed_from_end: Option<u8>,
 ) -> Result<GeneralFunctionShapeInfo> {
     GeneralFunctionSimpleSpec {
         name: "".to_owned(),
@@ -222,7 +223,7 @@ impl SpecTrait for GeneralFunctionSimpleSpec {
                 expected: 1
             });
         }
-        if shapes[0].len() < self.num_non_batchable_output_dims + self.removed_from_end {
+        if (shapes[0].len() as u8) < self.num_non_batchable_output_dims + self.removed_from_end {
             bail!(GeneralFunctionShapeError::NDimTooSmall {
                 ndim: shapes[0].len(),
                 num_non_batchable_output_dims: self.num_non_batchable_output_dims,
@@ -230,12 +231,12 @@ impl SpecTrait for GeneralFunctionSimpleSpec {
             });
         }
         Ok(GeneralFunctionShapeInfo {
-            shape: shapes[0][..shapes[0].len() - self.removed_from_end]
+            shape: shapes[0][..shapes[0].len() - self.removed_from_end as usize]
                 .iter()
                 .cloned()
                 .collect(),
             num_non_batchable_output_dims: self.num_non_batchable_output_dims,
-            input_batchability: vec![true],
+            input_batchability: vec![true].into_iter().collect(),
         })
     }
 
@@ -351,8 +352,8 @@ impl SpecTrait for GeneralFunctionIndexSpec {
                     .chain(&x_shape[final_index_dim + 1..])
                     .cloned()
                     .collect(),
-                num_non_batchable_output_dims: suffix_len - 1, // sub 1 for indexed
-                input_batchability: vec![true, true],
+                num_non_batchable_output_dims: (suffix_len - 1) as u8, // sub 1 for indexed
+                input_batchability: vec![true, true].into_iter().collect(),
             })
         } else {
             let index_dim = check_canon_idxs(x_shape.len(), &[self.index_dim])
@@ -365,8 +366,8 @@ impl SpecTrait for GeneralFunctionIndexSpec {
                     .chain(&x_shape[index_dim + 1..])
                     .cloned()
                     .collect(),
-                num_non_batchable_output_dims: x_shape.len() - 1, // sub 1 for indexed
-                input_batchability: vec![false, true],
+                num_non_batchable_output_dims: (x_shape.len() - 1) as u8, // sub 1 for indexed
+                input_batchability: vec![false, true].into_iter().collect(),
             })
         }
     }
@@ -556,9 +557,8 @@ pub struct GeneralFunction {
     info: CachedCircuitInfo,
     name: Option<String>,
     #[pyo3(get)]
-    pub num_non_batchable_output_dims: usize,
-    #[pyo3(get)]
-    pub input_batchability: Vec<bool>,
+    pub num_non_batchable_output_dims: u8,
+    pub input_batchability: TinyBoundedBitvec,
 }
 
 impl GeneralFunction {
@@ -602,7 +602,7 @@ impl GeneralFunction {
     }
 
     pub fn is_batchable(&self) -> bool {
-        self.input_batchability.iter().any(|x| *x)
+        self.input_batchability.into_iter().any(|x| x)
     }
 }
 
@@ -643,14 +643,14 @@ impl CircuitNode for GeneralFunction {
     }
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
-        let num_batchable_axes = self.info().rank() - self.num_non_batchable_output_dims;
-        zip(&self.nodes, &self.input_batchability)
+        let num_batchable_axes = self.info().rank() as u8 - self.num_non_batchable_output_dims;
+        zip(&self.nodes, self.input_batchability)
             .map(|(child, batchable)| {
                 if !batchable {
                     vec![None; child.info().rank()]
                 } else {
                     (0..child.info().rank())
-                        .map(|i| match i < num_batchable_axes {
+                        .map(|i| match i < num_batchable_axes as usize {
                             true => Some(i),
                             false => None,
                         })
@@ -897,7 +897,7 @@ impl GeneralFunctionSpecTester {
             }
         };
 
-        if num_non_batchable_output_dims > shape.len() {
+        if num_non_batchable_output_dims as usize > shape.len() {
             bail!(
             "too many non batchable output dims! num_non_batchable_output_dims={} shape.len()={}",
             num_non_batchable_output_dims,
@@ -912,14 +912,14 @@ impl GeneralFunctionSpecTester {
             );
         }
 
-        if input_batchability.iter().all(|x| !x) {
+        if input_batchability.into_iter().all(|x| !x) {
             // if none batchable, we don't have any tests to run
             return Ok((true, false));
         }
-        let current_num_batch_dims = shape.len() - num_non_batchable_output_dims;
+        let current_num_batch_dims = shape.len() - num_non_batchable_output_dims as usize;
 
-        for (shape, &is_batch) in
-            std::iter::once((&shape, &true)).chain(shapes.iter().zip(&input_batchability))
+        for (shape, is_batch) in
+            std::iter::once((&shape, true)).chain(shapes.iter().zip(input_batchability))
         {
             if is_batch && shape.len() < current_num_batch_dims {
                 bail!(
@@ -934,7 +934,7 @@ impl GeneralFunctionSpecTester {
             .chain(
                 shapes
                     .iter()
-                    .zip(&input_batchability)
+                    .zip(input_batchability)
                     .filter_map(|(s, is_batch)| is_batch.then(|| &s[..current_num_batch_dims])),
             )
             .collect();
@@ -954,8 +954,8 @@ impl GeneralFunctionSpecTester {
 
             let new_shapes: Vec<Shape> = shapes
                 .iter()
-                .zip(&input_batchability)
-                .map(|(s, &is_batch)| {
+                .zip(input_batchability)
+                .map(|(s, is_batch)| {
                     if is_batch {
                         batch_shape
                             .iter()
@@ -1080,8 +1080,8 @@ impl GeneralFunctionSpecTester {
                     .function(
                         &tensors
                             .iter()
-                            .zip(&input_batchability)
-                            .map(|(x, &b)| if b { run_idx(x.clone()) } else { x.clone() })
+                            .zip(input_batchability)
+                            .map(|(x, b)| if b { run_idx(x.clone()) } else { x.clone() })
                             .collect::<Vec<_>>(),
                         &Default::default(),
                     )
@@ -1302,8 +1302,8 @@ pub enum GeneralFunctionShapeError {
     #[error("ndim={ndim} < num_non_batchable_output_dims={num_non_batchable_output_dims} + removed_from_end={removed_from_end} ({e_name})")]
     NDimTooSmall {
         ndim: usize,
-        num_non_batchable_output_dims: usize,
-        removed_from_end: usize,
+        num_non_batchable_output_dims: u8,
+        removed_from_end: u8,
     },
 
     #[error("x_shape={x_shape:?} index_shape={index_shape:?} batch_x={batch_x} ({e_name})")]
