@@ -40,6 +40,7 @@ use rr_util::{
     sv,
     symbolic_size::{SymbolicSizeConstraints, SymbolicSizeProduct},
     tensor_util::{check_canon_idxs, Shape, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp},
+    tu8v,
     util::{
         arc_ref_clone, arc_unwrap_or_clone, AsOp, EinsumAxes, HashBytes, HashBytesToPy, NamedAxes,
     },
@@ -369,7 +370,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
     {
         let total_size: usize = check_canon_axes_as(self.info().rank(), axes)?
             .into_iter()
-            .map(|i| self.info().shape[i as usize])
+            .map(|i| self.info().shape[*i as usize])
             .product();
         let sum = self.sum(axes, None)?;
         Ok(Einsum::new(
@@ -381,7 +382,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
                         sv![],
                         scalar_name.or_else(|| self.name().map(|s| format!("mean_div_{}", s))),
                     ),
-                    sv![],
+                    tu8v![],
                 )))
                 .collect(),
             sum.out_axes,
@@ -509,9 +510,9 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
         let mut counter = 0;
         for i in 0..self.info().rank() as u8 {
             if axes.iter().contains(&i) {
-                input_ints.push(sv![])
+                input_ints.push(tu8v![])
             } else {
-                input_ints.push(sv![counter]);
+                input_ints.push(tu8v![counter]);
                 counter += 1;
             }
         }
@@ -519,7 +520,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
             self.crc(),
             RearrangeSpec {
                 input_ints,
-                output_ints: (0..num_ints).map(|i| sv![i as u8]).collect(),
+                output_ints: (0..num_ints).map(|i| tu8v![i as u8]).collect(),
                 int_sizes: sv![OpSize::NONE;num_ints],
             },
             name,
