@@ -1,31 +1,41 @@
-use std::iter::zip;
+use std::{
+    hash::{Hash, Hasher},
+    iter::{once, zip},
+    sync::{Arc, Weak},
+};
 
 use anyhow::{bail, Context, Result};
 use macro_rules_attribute::apply;
-use pyo3::prelude::*;
+use pyo3::{exceptions::PyValueError, prelude::*, pyclass::CompareOp};
 use rr_util::{
-    pycall,
+    cached_method,
+    py_types::use_rust_comp,
+    pycall, python_error_exception,
+    rearrange_spec::RearrangeSpec,
     symbolic_size::SymbolicSizeProduct,
-    tensor_util::{Shape, TorchDeviceDtype},
-    util::{counts_g_1, is_unique},
+    tensor_util::{right_align_shapes, Shape, TorchDeviceDtype},
+    util::{counts_g_1, is_unique, unique_to_appearance},
 };
-use rustc_hash::FxHashMap as HashMap;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet, FxHasher};
+use thiserror::Error;
 use uuid::{uuid, Uuid};
 
 use crate::{
-    all_children, circuit_node_auto_impl, circuit_node_extra_impl,
+    circuit_node_auto_impl, circuit_node_extra_impl,
     circuit_node_private::{CircuitNodeHashItems, CircuitNodePrivate, CircuitNodeSetNonHashInfo},
     circuit_utils::OperatorPriority,
-    deep_map, deep_map_op_context_preorder_stoppable, deep_map_unwrap,
+    deep_map, deep_map_op_context_preorder_stoppable, deep_map_preorder_unwrap, deep_map_unwrap,
     expand_node::{replace_expand_bottom_up, ExpandError},
+    named_axes::{deep_strip_axis_names, propagate_named_axes},
     new_rc_unwrap,
     prelude::*,
-    visit_circuit_unwrap, CachedCircuitInfo, HashBytes, PyCircuitBase, Symbol, TensorEvalError,
+    visit_circuit, visit_circuit_unwrap, CachedCircuitInfo, HashBytes, PyCircuitBase, Rearrange,
+    Symbol, TensorEvalError,
 };
 
 /// can also be thought of as lambda from lambda calculus (expression with free variables + list of these variables)
 #[pyclass]
-#[derive(Debug, Clone, Hash, PartialEq, Eq)]
+#[derive(Debug, Clone, Hash, PartialEq, Eq, PartialOrd, Ord)]
 pub struct ModuleSpec {
     #[pyo3(get, set)]
     pub circuit: CircuitRc,
@@ -33,8 +43,33 @@ pub struct ModuleSpec {
     pub arg_specs: Vec<ModuleArgSpec>,
 }
 
+type ModuleSpecHashable = (HashBytes, Vec<ModuleArgSpecHashable>);
+
 impl ModuleSpec {
     pub const EXPAND_PLACEHOLDER_UUID: Uuid = uuid!("741ba404-eec3-4ac9-b6ce-062e903fb033");
+
+    pub fn get_hashable(&self) -> ModuleSpecHashable {
+        (
+            self.circuit.info().hash,
+            self.arg_specs.iter().map(|x| x.get_hashable()).collect(),
+        )
+    }
+
+    pub fn get_spec_circuit_uuid(&self) -> Uuid {
+        let x: [_; 16] = self.circuit.info().hash[..16].try_into().unwrap();
+        Uuid::from_bytes(x)
+    }
+
+    pub fn batch_shapes<'a>(&self, nodes: &'a [CircuitRc]) -> Vec<&'a [usize]> {
+        zip(self.arg_specs.iter(), nodes)
+            .map(|(arg_spec, node)| &node.shape()[..node.ndim() - arg_spec.symbol.ndim()])
+            .collect()
+    }
+
+    pub fn aligned_batch_shape(&self, nodes: &[CircuitRc]) -> Result<Shape> {
+        right_align_shapes(&self.batch_shapes(nodes))
+    }
+
     pub fn expand_raw(&self, nodes: &Vec<CircuitRc>) -> Result<CircuitRc> {
         if self.arg_specs.len() != nodes.len() {
             bail!(ConstructError::ModuleWrongNumberChildren {
@@ -93,18 +128,49 @@ impl ModuleSpec {
                 }
             }
         }
-        replace_expand_bottom_up(self.circuit.clone(), |c| {
+
+        // TODO: maybe we should allow for inconsistent symbolic batch shapes?
+        let aligned_batch_shape = self
+            .aligned_batch_shape(nodes)
+            .context("batch shapes didn't match for module")?;
+
+        let out = replace_expand_bottom_up(self.circuit.clone(), |c| {
             self.arg_specs
                 .iter()
                 .position(|x| x.symbol.info().hash == c.info().hash)
                 .map(|i| nodes[i].clone())
-        })
+        })?;
+
+        assert!(out.ndim() >= self.circuit.ndim());
+        let out_batch_rank = out.ndim() - self.circuit.ndim();
+        assert!(out_batch_rank <= aligned_batch_shape.len());
+        right_align_shapes(&[&out.shape()[..out_batch_rank], &aligned_batch_shape])
+            .expect("output shape should be right aligned subset of batch shape");
+
+        let out = if out_batch_rank < aligned_batch_shape.len() {
+            let extra_batch = aligned_batch_shape.len() - out_batch_rank;
+            let spec = RearrangeSpec::prepend_batch_shape(
+                aligned_batch_shape[..extra_batch].iter().cloned().collect(),
+                out.ndim(),
+            )
+            .context("rank overflow in module expand")?;
+
+            let rep_name = out.name().map(|x| format!("{} rep_batch", x));
+            Rearrange::nrc(out, spec, rep_name)
+        } else {
+            out
+        };
+        Ok(out)
     }
 
-    pub fn expand_shape(&self, shapes: &Vec<Shape>) -> Result<CircuitRc> {
-        if let Some(result) = MODULE_EXPANSIONS_SHAPE
-            .with(|cache| cache.borrow().get(&(self.clone(), shapes.clone())).cloned())
-        {
+    pub fn expand_shape(&self, shapes: &Vec<Shape>) -> Result<(CircuitRc, Vec<HashBytes>)> {
+        if let Some(result) = MODULE_EXPANSIONS_SHAPE.with(|cache| {
+            cache
+                .borrow()
+                .get(&(self.get_hashable(), shapes.clone()))
+                .map(|(w, shapes)| w.upgrade().map(|w| (CircuitRc(w), shapes.clone())))
+                .flatten()
+        }) {
             return Ok(result);
         }
         let symbols = shapes
@@ -113,18 +179,20 @@ impl ModuleSpec {
             .map(|(i, s)| {
                 Symbol::nrc(
                     s.clone(),
-                    ModuleSpec::EXPAND_PLACEHOLDER_UUID,
+                    self.get_spec_circuit_uuid(),
                     Some(format!("{}_{:?}", i, s)),
                 )
             })
             .collect();
         let result = self.expand_raw(&symbols)?;
+        let out_bytes: Vec<_> = symbols.into_iter().map(|x| x.info().hash).collect();
         MODULE_EXPANSIONS_SHAPE.with(|cache| {
-            cache
-                .borrow_mut()
-                .insert((self.clone(), shapes.clone()), result.clone())
+            cache.borrow_mut().insert(
+                (self.get_hashable(), shapes.clone()),
+                (Arc::downgrade(&result.0), out_bytes.clone()),
+            )
         });
-        Ok(result)
+        Ok((result, out_bytes))
     }
 
     pub fn substitute(
@@ -133,33 +201,53 @@ impl ModuleSpec {
         name_prefix: Option<String>,
         name_suffix: Option<String>,
     ) -> Result<CircuitRc> {
-        let key: (ModuleSpec, Vec<HashBytes>, Option<String>) = (
-            self.clone(),
+        let key: (_, Vec<HashBytes>, Option<String>) = (
+            self.get_hashable(),
             nodes.iter().map(|x| x.info().hash).collect(),
             name_prefix.clone(),
         );
 
-        if let Some(result) = MODULE_EXPANSIONS.with(|cache| cache.borrow().get(&key).cloned()) {
+        if let Some(result) = MODULE_EXPANSIONS.with(|cache| {
+            cache
+                .borrow()
+                .get(&key)
+                .map(|x| x.upgrade().map(CircuitRc))
+                .flatten()
+        }) {
             return Ok(result);
         }
+
+        for (arg_spec, sub_for_sym) in self.arg_specs.iter().zip(nodes) {
+            check_valid_sub(self.circuit.clone(), &arg_spec.symbol, sub_for_sym.clone())?
+        }
+        for idxs in unique_to_appearance(&self.arg_specs.iter().map(|x| x.symbol.ident()).collect())
+            .into_values()
+        {
+            if idxs.len() <= 1 {
+                continue;
+            }
+            if idxs
+                .iter()
+                .map(|i| &nodes[*i])
+                .collect::<HashSet<_>>()
+                .len()
+                > 1
+            {
+                bail!(SubstitutionError::BindsEquivalentSymbolsToDifferentNodes {
+                    symbols: self
+                        .arg_specs
+                        .iter()
+                        .map(|arg_spec| arg_spec.symbol.clone())
+                        .collect(),
+                    nodes: nodes.clone(),
+                    idxs
+                });
+            }
+        }
+
         let shapes = nodes.iter().map(|x| x.info().shape.clone()).collect();
-        let mut expanded_shape = self.expand_shape(&shapes)?;
-        let node_mapping: HashMap<HashBytes, CircuitRc> = nodes
-            .iter()
-            .enumerate()
-            .map(|(i, n)| {
-                (
-                    Symbol::new(
-                        n.info().shape.clone(),
-                        ModuleSpec::EXPAND_PLACEHOLDER_UUID,
-                        Some(format!("{}_{:?}", i, n.info().shape)),
-                    )
-                    .info()
-                    .hash,
-                    n.clone(),
-                )
-            })
-            .collect();
+        let (mut expanded_shape, rep_hashes) = self.expand_shape(&shapes)?;
+        let node_mapping: HashMap<_, _> = rep_hashes.into_iter().zip(nodes).collect();
         if name_prefix.is_some() || name_suffix.is_some() {
             expanded_shape = deep_map_unwrap(expanded_shape, |x| {
                 if let Some(n) = x.name() {
@@ -176,30 +264,37 @@ impl ModuleSpec {
         }
         // we're replacing leaves so no need to do anything with stoppable preorder
         let result = deep_map(expanded_shape, |circ| {
-            Ok(node_mapping.get(&circ.info().hash).cloned().unwrap_or(circ))
+            Ok(node_mapping
+                .get(&circ.info().hash)
+                .cloned()
+                .cloned()
+                .unwrap_or(circ))
         })
         .with_context(|| {
             format!(
                 concat!(
                     "replacing symbols with nodes",
                     " failed in ModuleSpec substitute\n",
-                    "common errors:\n",
-                    "  - Multiple dtypes (see below error)\n",
-                    "  - Replacing inputs symbols to a nest module with ",
-                    "non-symbols (nodes aren't necessary symbols)\n",
+                    "This is typically caused by multiple dtypes ",
+                    "(see root cause below)\n",
                     "module_spec={:?}\nnodes={:?}"
                 ),
                 self, nodes
             )
         })?;
-        MODULE_EXPANSIONS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
+        MODULE_EXPANSIONS.with(|cache| cache.borrow_mut().insert(key, Arc::downgrade(&result.0)));
         Ok(result)
     }
 
     pub fn compute_non_children_hash(&self, hasher: &mut blake3::Hasher) {
         for arg_spec in &self.arg_specs {
             // this is fine because each item is fixed size and we delimit with node hashs (which are uu)
-            hasher.update(&[arg_spec.batchable as u8, arg_spec.expandable as u8]);
+            hasher.update(&[
+                arg_spec.batchable as u8,
+                arg_spec.expandable as u8,
+                arg_spec.ban_non_symbolic_size_expand as u8,
+            ]);
+            hasher.update(&arg_spec.symbol.info().hash);
         }
     }
 
@@ -256,15 +351,15 @@ impl ModuleSpec {
     }
 
     fn check_all_args_present(&self) -> Result<()> {
-        let all_symbols = self.arg_specs.iter().map(|x| x.symbol.crc()).collect();
-        let children = all_children(self.circuit.clone());
-        if !children.is_superset(&all_symbols) {
+        let missing_symbols: HashSet<_> = self
+            .arg_specs
+            .iter()
+            .filter_map(|x| (!has_sym(&self.circuit, &x.symbol)).then(|| x.symbol.clone()))
+            .collect();
+        if !missing_symbols.is_empty() {
             bail!(ConstructError::ModuleSomeArgsNotPresent {
                 spec_circuit: self.circuit.clone(),
-                missing_symbols: all_symbols
-                    .difference(&children)
-                    .map(|x| x.as_symbol_unwrap().clone())
-                    .collect(),
+                missing_symbols,
             })
         }
         Ok(())
@@ -404,10 +499,20 @@ impl ModuleSpec {
     pub fn __repr__(&self) -> String {
         format!("{:?}", self)
     }
+
+    fn __richcmp__(&self, object: &Self, comp_op: CompareOp) -> bool {
+        use_rust_comp(&self, &object, comp_op)
+    }
+
+    fn __hash__(&self) -> u64 {
+        let mut s = FxHasher::default();
+        self.hash(&mut s);
+        s.finish()
+    }
 }
 
 #[pyclass]
-#[derive(Debug, Clone, Hash, PartialEq, Eq)]
+#[derive(Debug, Clone, Hash, PartialEq, Eq, PartialOrd, Ord)]
 pub struct ModuleArgSpec {
     #[pyo3(get, set)]
     pub symbol: Symbol,
@@ -430,6 +535,21 @@ impl Default for ModuleArgSpec {
     }
 }
 
+pub type ModuleArgSpecHashable = (HashBytes, [bool; 3]);
+
+impl ModuleArgSpec {
+    pub fn get_hashable(&self) -> ModuleArgSpecHashable {
+        (
+            self.symbol.info().hash,
+            [
+                self.batchable,
+                self.expandable,
+                self.ban_non_symbolic_size_expand,
+            ],
+        )
+    }
+}
+
 #[pymethods]
 impl ModuleArgSpec {
     #[new]
@@ -477,6 +597,16 @@ impl ModuleArgSpec {
     pub fn __repr__(&self) -> String {
         format!("{:?}", self)
     }
+
+    fn __richcmp__(&self, object: &Self, comp_op: CompareOp) -> bool {
+        use_rust_comp(&self, &object, comp_op)
+    }
+
+    fn __hash__(&self) -> u64 {
+        let mut s = FxHasher::default();
+        self.hash(&mut s);
+        s.finish()
+    }
 }
 
 /// can also be thought of as a lambda + it's arguments in lambda calculus (but not yet beta reduced)
@@ -499,7 +629,7 @@ impl Module {
     pub fn try_new(nodes: Vec<CircuitRc>, spec: ModuleSpec, name: Option<String>) -> Result<Self> {
         let cached_substituted = spec
             .substitute(&nodes, None, None)
-            .with_context(|| format!("module construction failed on expand name={:?}", name))?;
+            .with_context(|| format!("module construction failed on substitute name={:?}", name))?;
 
         let mut out = Self {
             nodes,
@@ -555,6 +685,42 @@ impl Module {
 
         Self::try_new(nodes.into_iter().map(Option::unwrap).collect(), spec, name)
     }
+
+    fn child_axis_map_inputs_uncached(&self) -> Result<Vec<Vec<Option<usize>>>> {
+        let spec_resized = self
+            .spec
+            .resize(self.nodes.iter().map(|x| x.info().shape.clone()).collect())?;
+        let spec_circuit_resized = spec_resized.circuit.clone();
+        assert!(spec_circuit_resized.info().shape[..] == self.info().shape[..]);
+        let stripped = deep_strip_axis_names(spec_circuit_resized, &None);
+        let out_named = propagate_named_axes(
+            stripped,
+            (0..self.info().rank())
+                .map(|i| (i as u8, i.to_string()))
+                .collect(),
+            true,
+        );
+        let mut result: Vec<Vec<Option<usize>>> = self
+            .nodes
+            .iter()
+            .map(|c| vec![None; c.info().rank()])
+            .collect();
+        visit_circuit_unwrap(out_named, |x| {
+            if let Some(sym) = x.as_symbol() {
+                if let Some(i) = self
+                    .spec
+                    .arg_specs
+                    .iter()
+                    .position(|x| x.symbol.uuid == sym.uuid && x.symbol.name() == sym.name())
+                {
+                    for (k, v) in &sym.info().named_axes {
+                        result[i][*k as usize] = Some(v.parse::<usize>().unwrap());
+                    }
+                }
+            }
+        });
+        Ok(result)
+    }
 }
 
 circuit_node_extra_impl!(Module, self_hash_default);
@@ -576,22 +742,14 @@ impl CircuitNode for Module {
     circuit_node_auto_impl!("6825f723-f178-4dab-b568-cd85eb6d2bf3");
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
-        // TODO: this can be done by first stripping named axes and then
-        // propagating named axes up from symbols in spec circuit
-        unimplemented!();
+        let map_inputs = self.child_axis_map_inputs().unwrap();
+        once(vec![None; self.spec.circuit.info().rank()])
+            .chain(map_inputs)
+            .collect()
     }
 
     fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(
-            std::iter::once(self.spec.circuit.clone()).chain(
-                self.spec
-                    .arg_specs
-                    .iter()
-                    // zip so we get the right ordering (same as map_children_enumerate)
-                    .zip(self.nodes.iter().cloned())
-                    .flat_map(|(arg_spec, n)| [arg_spec.symbol.crc(), n]),
-            ),
-        )
+        Box::new(std::iter::once(self.spec.circuit.clone()).chain(self.nodes.clone()))
     }
 
     fn non_free_children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
@@ -610,35 +768,19 @@ impl CircuitNode for Module {
             "guaranteed by constructor via expand"
         );
 
-        // map both symbols and nodes
-        let out = self
-            .spec
-            .arg_specs
+        let nodes = self
+            .nodes
             .clone()
             .into_iter()
-            .zip(&self.nodes)
             .enumerate()
-            .map(|(i, (arg_spec, node))| {
-                let maybe_new_sym = f(2 * i + 1, arg_spec.symbol.rc())?;
-                let symbol = maybe_new_sym.as_symbol().cloned().ok_or_else(|| {
-                    ConstructError::ModuleExpectedSymbolOnMap {
-                        orig_module: self.clone(),
-                        got_instead_of_symbol: maybe_new_sym,
-                    }
-                })?;
-                let new_arg = ModuleArgSpec { symbol, ..arg_spec };
-                let new_node = f(2 * i + 2, node.clone())?;
-                Ok((new_arg, new_node))
-            })
-            .collect::<Result<Vec<_>>>()?;
-
-        let (arg_specs, nodes) = out.into_iter().unzip();
+            .map(|(i, node)| f(i + 1, node))
+            .collect::<Result<_>>()?;
 
         Self::try_new(
             nodes,
             ModuleSpec {
                 circuit: new_spec_circ,
-                arg_specs,
+                arg_specs: self.spec.arg_specs.clone(),
             },
             self.name.clone(),
         )
@@ -741,6 +883,46 @@ impl Module {
         Ok(self.substitute(name_prefix, name_suffix))
     }
 
+    pub fn aligned_batch_shape(&self) -> Shape {
+        self.spec.aligned_batch_shape(&self.nodes).unwrap()
+    }
+
+    /// None is conform all dims
+    pub fn conform_to_input_batch_shape(&self, dims_to_conform: Option<usize>) -> Result<Self> {
+        let current_aligned = self.aligned_batch_shape();
+        let dims_to_conform = dims_to_conform.unwrap_or(current_aligned.len());
+
+        if dims_to_conform > current_aligned.len() {
+            // better error as needed
+            bail!("dims to conform is larger than num batch dims!")
+        }
+
+        let batch_shapes = self.spec.batch_shapes(&self.nodes);
+
+        Ok(Self::new(
+            self.nodes.clone(),
+            self.spec
+                .resize(
+                    self.spec
+                        .arg_specs
+                        .iter()
+                        .zip(&batch_shapes)
+                        .map(|(arg_spec, current_batch_shape)| {
+                            let batch_start =
+                                current_batch_shape.len().saturating_sub(dims_to_conform);
+                            current_batch_shape[batch_start..]
+                                .iter()
+                                .chain(arg_spec.symbol.shape())
+                                .copied()
+                                .collect()
+                        })
+                        .collect(),
+                )
+                .expect("constructor should ensure this works"),
+            self.name_cloned(),
+        ))
+    }
+
     // TODO: add some naming options maybe
     pub fn conform_to_input_shapes(&self) -> Self {
         Self::new(
@@ -751,6 +933,30 @@ impl Module {
             self.name_cloned(),
         )
     }
+
+    fn child_axis_map_inputs(&self) -> Result<Vec<Vec<Option<usize>>>> {
+        let key = (
+            self.spec.get_hashable(),
+            self.nodes.iter().map(|x| x.info().shape.clone()).collect(),
+        );
+        if let Some(result) =
+            MODULE_EXPANSIONS_AXIS_MAPS.with(|cache| cache.borrow().get(&key).cloned())
+        {
+            return Ok(result);
+        }
+
+        let result = self.child_axis_map_inputs_uncached()?;
+        MODULE_EXPANSIONS_AXIS_MAPS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
+        Ok(result)
+    }
+
+    pub fn arg_items(&self) -> Vec<(CircuitRc, ModuleArgSpec)> {
+        self.nodes
+            .clone()
+            .into_iter()
+            .zip(self.spec.arg_specs.clone())
+            .collect()
+    }
 }
 
 impl Module {
@@ -780,17 +986,162 @@ pub fn substitute_all_modules(circuit: CircuitRc) -> CircuitRc {
 
 #[pyfunction]
 pub fn conform_all_modules(circuit: CircuitRc) -> CircuitRc {
-    deep_map_unwrap(circuit, |c| match &**c {
+    // this has to be preorder so that we resolve to outermost sizes
+    deep_map_preorder_unwrap(circuit, |c| match &**c {
         Circuit::Module(mn) => mn.conform_to_input_shapes().rc(),
         _ => c.clone(),
     })
 }
 
+#[pyfunction]
+pub fn get_children_with_symbolic_sizes(circuit: CircuitRc) -> HashSet<CircuitRc> {
+    let mut circuits_with_symbolic_sizes = HashSet::default();
+    visit_circuit_unwrap(circuit, |x| {
+        if x.shape()
+            .iter()
+            .any(|s| SymbolicSizeProduct::has_symbolic(*s))
+        {
+            circuits_with_symbolic_sizes.insert(x);
+        }
+    });
+    circuits_with_symbolic_sizes
+}
+
+#[pyfunction]
+pub fn any_children_with_symbolic_sizes(circuit: CircuitRc) -> bool {
+    !get_children_with_symbolic_sizes(circuit).is_empty()
+}
+
+// adhoc anyfound
+// TODO: do this batchedly over symbols while still caching individually?
+#[derive(Clone, Debug)]
+pub struct HasSym {
+    cache: cached::UnboundCache<(HashBytes, HashBytes, bool), bool>,
+}
+impl Default for HasSym {
+    fn default() -> Self {
+        Self {
+            cache: cached::UnboundCache::new(),
+        }
+    }
+}
+impl HasSym {
+    #[apply(cached_method)]
+    #[self_id(self_)]
+    #[key((circuit.info().hash, sym.info().hash, by_ident))]
+    #[cache_expr(cache)]
+    pub fn has_sym(&mut self, circuit: &CircuitRc, sym: &Symbol, by_ident: bool) -> bool {
+        if (by_ident && circuit.as_symbol().map(|x| x.ident()) == Some(sym.ident()))
+            || (!by_ident && circuit.as_symbol() == Some(sym))
+        {
+            true
+        } else {
+            circuit.children().any(|c| self_.has_sym(&c, sym, by_ident))
+        }
+    }
+}
+
+/// uses global cache
+pub fn has_sym_impl(circuit: &CircuitRc, sym: &Symbol, by_ident: bool) -> bool {
+    HAS_SYM.with(|has_sym| has_sym.borrow_mut().has_sym(circuit, sym, by_ident))
+}
+pub fn has_sym(circuit: &CircuitRc, sym: &Symbol) -> bool {
+    has_sym_impl(circuit, sym, false)
+}
+pub fn has_sym_ident(circuit: &CircuitRc, sym: &Symbol) -> bool {
+    has_sym_impl(circuit, sym, true)
+}
+
+pub fn check_valid_sub(circuit: CircuitRc, sym: &Symbol, sub_for_sym: CircuitRc) -> Result<()> {
+    let hashs = [
+        circuit.info().hash,
+        sym.info().hash,
+        sub_for_sym.info().hash,
+    ];
+    if IS_VALID_SUB.with(|is_valid| is_valid.borrow().contains(&hashs)) {
+        return Ok(());
+    }
+
+    let ok = || {
+        IS_VALID_SUB.with(|is_valid| is_valid.borrow_mut().insert(hashs));
+    };
+
+    // we could do more caching I guess, but this seems fine.
+    let free: HashSet<_> = all_free(sub_for_sym.clone()).into_iter().collect();
+    let free_idents: HashSet<_> = free.iter().map(|x| x.ident()).collect();
+
+    visit_circuit(circuit, |x| {
+        if let Some(m) = x.as_module() {
+            if has_sym_ident(&m.spec.circuit, sym) {
+                for (arg_spec, node) in m.spec.arg_specs.iter().zip(&m.nodes) {
+                    if arg_spec.symbol.ident() == sym.ident() && node != &sub_for_sym {
+                        bail!(
+                            SubstitutionError::BindsSameSymbolToDifferentValueThanNestedModule {
+                                circ: sub_for_sym.clone(),
+                                sym: sym.clone(),
+                                nested_module: m.clone(),
+                                different_circ: node.clone()
+                            }
+                        );
+                    }
+                    if free_idents.contains(&arg_spec.symbol.ident()) {
+                        bail!(SubstitutionError::CircuitHasFreeSymBoundByNestedModule {
+                            circ: sub_for_sym.clone(),
+                            sym: sym.clone(),
+                            nested_module: m.clone(),
+                            free_sym: arg_spec.symbol.clone()
+                        });
+                    }
+                }
+            }
+        }
+        Ok(())
+    })?;
+    ok();
+    Ok(())
+}
+
+#[apply(python_error_exception)]
+#[base_error_name(Substitution)]
+#[base_exception(PyValueError)]
+#[derive(Error, Debug, Clone)]
+pub enum SubstitutionError {
+    #[error("subbing circ={circ:?} for sym={sym:?} inconsistent with nested_module={nested_module:?} which binds equivalent sym to different_circ={different_circ:?} ({e_name})")]
+    BindsSameSymbolToDifferentValueThanNestedModule {
+        circ: CircuitRc,
+        sym: Symbol,
+        nested_module: Module,
+        different_circ: CircuitRc,
+    },
+
+    #[error("subbing circ={circ:?} for sym={sym:?} results in nested_module={nested_module:?} replacing free_sym={free_sym:?} {} ({e_name})",
+        "inside of circ (aka higher order function)\nNote: it may be equivalent ignoring shape\n")]
+    CircuitHasFreeSymBoundByNestedModule {
+        circ: CircuitRc,
+        sym: Symbol,
+        nested_module: Module,
+        free_sym: Symbol,
+    },
+
+    #[error("symbols={symbols:?} nodes={nodes:?} idxs={idxs:?} at these idxs, symbols are equivalent, but have different nodes {} ({e_name})",
+        "(equivalence ignores shape)")]
+    BindsEquivalentSymbolsToDifferentNodes {
+        symbols: Vec<Symbol>,
+        nodes: Vec<CircuitRc>,
+        idxs: Vec<usize>,
+    },
+}
+
 use std::cell::RefCell;
 // TODO: maybe cache check_unique_arg_names here?
 thread_local! {
-    static MODULE_EXPANSIONS: RefCell<HashMap<(ModuleSpec, Vec<HashBytes>, Option<String>), CircuitRc>> =
+    static MODULE_EXPANSIONS: RefCell<HashMap<(ModuleSpecHashable, Vec<HashBytes>, Option<String>), Weak<Circuit>>> =
         RefCell::new(HashMap::default());
-    static MODULE_EXPANSIONS_SHAPE: RefCell<HashMap<(ModuleSpec, Vec<Shape>), CircuitRc>> =
+    static MODULE_EXPANSIONS_SHAPE: RefCell<HashMap<(ModuleSpecHashable, Vec<Shape>), (Weak<Circuit>, Vec<HashBytes>)>> =
         RefCell::new(HashMap::default());
+    static MODULE_EXPANSIONS_AXIS_MAPS: RefCell<HashMap<(ModuleSpecHashable, Vec<Shape>), Vec<Vec<Option<usize>>> >> =
+        RefCell::new(HashMap::default());
+    static IS_VALID_SUB: RefCell<HashSet<[HashBytes; 3]>> =
+        RefCell::new(Default::default());
+    static HAS_SYM: RefCell<HasSym>= RefCell::new(Default::default())
 }
