@@ -1,6 +1,6 @@
 use std::collections::BTreeMap;
 
-use anyhow::{bail, Result};
+use anyhow::{bail, Context, Result};
 use circuit_base::{
     deep_map_op, deep_map_unwrap,
     named_axes::{axis_of_name, get_axis_leaves, set_named_axes},
@@ -46,22 +46,23 @@ pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> R
     // for k in &leaves_non_axis {
     //     k.print();
     // }
-    let input_specs_axis: Vec<(CircuitRc, ModuleArgSpec)> = leaves_axis
+    let input_specs_axis = leaves_axis
         .iter()
         .map(|(sub, i)| {
-            (
+            Ok((
                 sub.clone(),
                 ModuleArgSpec {
                     symbol: set_named_axes(
                         Symbol::new_with_random_uuid(sub.info().shape.clone(), None),
                         BTreeMap::from([(*i as u8, name.clone())]),
-                    ),
+                    )
+                    .context("Failed extracting named axis from old name.")?,
                     ban_non_symbolic_size_expand: false, // I think this needs to be false?
                     ..Default::default()
                 },
-            )
+            ))
         })
-        .collect();
+        .collect::<Result<Vec<(CircuitRc, ModuleArgSpec)>>>()?;
     let input_specs: Vec<(CircuitRc, ModuleArgSpec)> = input_specs_axis
         .iter()
         .cloned()
@@ -211,6 +212,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> Resul
                         .map(|(i, int)| (i as u8, int.to_string()))
                         .collect(),
                 )
+                .unwrap() // u8 don't have spaces
                 .rc(),
                 ints.clone(),
             )
