@@ -30,7 +30,7 @@ use crate::{
         generalfunction_evaluate_simple, generalfunction_merge_inverses,
         generalfunction_special_case_simplification,
     },
-    module_rewrite::elim_empty_module,
+    module_rewrite::{elim_empty_module, elim_no_input_module},
     nb_rewrites::{add_elim_removable_axes_weak, einsum_elim_removable_axes_weak},
     scatter_rewrite::{
         add_pull_scatter, concat_to_scatter, einsum_pull_scatter, index_einsum_to_scatter,
@@ -155,7 +155,7 @@ fn all_simp_fns() -> SimpFns {
             f_wrap!(add_pull_scatter),
             f_wrap!(add_pull_diags),
             f_wrap!(add_fuse_scalar_multiples),
-            f_wrap!(add_elim_removable_axes_weak), // TODO: I don't understand this
+            f_wrap!(add_elim_removable_axes_weak),
         ],
         einsum: vec![
             f_wrap!(einsum_elim_zero),
@@ -164,7 +164,7 @@ fn all_simp_fns() -> SimpFns {
             f_wrap!(einsum_of_permute_merge),
             f_wrap!(einsum_merge_scalars),
             l_wrap!(einsum_pull_removable_axes),
-            f_wrap!(einsum_elim_removable_axes_weak), // TODO: maybe understand this
+            f_wrap!(einsum_elim_removable_axes_weak),
             f_wrap!(einsum_permute_to_rearrange),
             l_wrap!(einsum_pull_scatter),
             f_wrap!(einsum_push_down_trace),
@@ -204,7 +204,7 @@ fn all_simp_fns() -> SimpFns {
             f_wrap!(scatter_fuse),
             f_wrap!(scatter_pull_removable_axes),
         ],
-        module: vec![l_wrap!(elim_empty_module)],
+        module: vec![l_wrap!(elim_empty_module), l_wrap!(elim_no_input_module)],
         ..Default::default()
     }
 }
@@ -237,6 +237,8 @@ impl Default for SimpFnSubset {
             "concat_drop_size_zero",
             "concat_fuse",
             "concat_repeat_to_rearrange",
+            "elim_empty_module",
+            "elim_no_input_module",
         ]
         .into_iter()
         .map(|x| x.to_owned())
