@@ -174,7 +174,7 @@ impl Dag {
     }
 
     // warning: I'm modifying the graph as I'm traversing it. edits must not mess with topo frontier
-    pub fn simplify_pass(&mut self, simp_settings: &DagSimpSettings, do_isolated: bool) -> bool {
+    pub fn simplify_pass(&mut self, simp_settings: &DagSimpSettings, _do_isolated: bool) -> bool {
         // simplify bottom_up
         let mut did_anything = false;
 
