@@ -4,6 +4,7 @@
 pub mod iterative_matcher;
 pub mod library;
 pub mod matcher;
+pub mod matcher_debug;
 pub mod operations;
 pub mod sampler;
 pub mod transform;
@@ -12,5 +13,7 @@ pub use iterative_matcher::{
     IterateMatchResults, IterativeMatcher, IterativeMatcherData, IterativeMatcherRc,
 };
 pub use matcher::{Matcher, MatcherData, MatcherFromPy, MatcherFromPyBase, MatcherRc};
-pub use operations::{BoundGetter, BoundUpdater, Expander, Getter, Updater};
+pub use operations::{
+    AnyFound, BoundAnyFound, BoundGetter, BoundUpdater, Expander, Getter, Updater,
+};
 pub use transform::{Transform, TransformData, TransformRc};
