@@ -23,6 +23,7 @@ use rr_util::{
     tensor_util::{
         Shape, Slice, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp,
     },
+    tu8v,
     util::{flip_op_result, HashBytes},
 };
 use smallvec::SmallVec as Sv;
@@ -475,9 +476,9 @@ impl SampleSpecTrait for RunDiscreteVarAllSpec {
         let all_weights_sum = Einsum::nrc(
             vec![(
                 all_weights.clone(),
-                Sv::from_iter(0..(all_weights.info().rank() as u8)),
+                (0..(all_weights.info().rank() as u8)).collect(),
             )],
-            Sv::default(),
+            tu8v![],
             None,
         );
         all_weights
@@ -495,9 +496,9 @@ impl SampleSpecTrait for RunDiscreteVarAllSpec {
         Einsum::nrc(
             vec![(
                 self.get_unnorm_empirical_weights(),
-                Sv::from_iter(0..(self.get_sample_shape().len() as u8)),
+                (0..(self.get_sample_shape().len() as u8)).collect(),
             )],
-            Sv::default(),
+            tu8v![],
             None,
         )
     }
@@ -688,9 +689,9 @@ impl Sampler {
         Einsum::try_new(
             vec![(
                 self.sample_spec.get_empirical_weights(),
-                Sv::from_iter(0..spec_nb_axes),
+                (0..spec_nb_axes).collect(),
             )],
-            Sv::default(),
+            tu8v![],
             None,
         )
         .unwrap()
@@ -719,10 +720,10 @@ impl Sampler {
         });
         Ok(Einsum::try_new(
             vec![
-                (self.normed_weights(), Sv::from_iter(0..spec_nb_axes)),
-                (circuit, Sv::from_iter(0..circuit_rank)),
+                (self.normed_weights(), (0..spec_nb_axes).collect()),
+                (circuit, (0..circuit_rank).collect()),
             ],
-            Sv::from_iter(spec_nb_axes..circuit_rank),
+            (spec_nb_axes..circuit_rank).collect(),
             name,
         )?
         .rc())
