@@ -2,13 +2,14 @@ use std::{fmt::Write, iter::zip};
 
 use anyhow::Result;
 use circuit_base::{
-    print::{color, PrintOptions},
+    clicolor,
+    print::{color, CliColor, PrintOptions},
     CircuitNode, CircuitNodeSelfOnlyHash, CircuitRc,
 };
+use once_cell::unsync::Lazy;
 use pyo3::prelude::*;
 use rr_util::util::{indent, HashBytes};
 use uuid::uuid;
-
 #[pyfunction(
     hash_child_count = "true",
     hash_child_shapes = "false",
@@ -46,10 +47,10 @@ pub fn compute_self_hash(
     m.finalize().into()
 }
 
-const SAME_SELF_COLOR: Option<usize> = Some(3);
-const SAME_COLOR: Option<usize> = None;
-const NEW_COLOR: Option<usize> = Some(1);
-const REMOVED_COLOR: Option<usize> = Some(0);
+const SAME_SELF_COLOR: Lazy<CliColor> = Lazy::new(|| clicolor!(Blue));
+const SAME_COLOR: CliColor = CliColor::NONE;
+const NEW_COLOR: Lazy<CliColor> = Lazy::new(|| clicolor!(Green));
+const REMOVED_COLOR: Lazy<CliColor> = Lazy::new(|| clicolor!(Red));
 
 #[pyfunction(
     options = "Default::default()",
@@ -110,7 +111,7 @@ pub fn diff_circuits(
             result.push_str(&indent(
                 color(
                     &format!("{}{}", line_prefix, options.repr_line_info(new.clone())?),
-                    SAME_SELF_COLOR,
+                    *SAME_SELF_COLOR,
                 ),
                 last_child_stack.len() * 2,
             ));
@@ -137,7 +138,7 @@ pub fn diff_circuits(
             return Ok(());
         }
         let mut new_options = options.clone();
-        new_options.colorer = Some(PrintOptions::fixed_color(NEW_COLOR));
+        new_options.colorer = Some(PrintOptions::fixed_color(*NEW_COLOR));
         result.push_str(&indent(
             new_options.repr(new.clone())?,
             last_child_stack.len() * 2,
@@ -145,7 +146,7 @@ pub fn diff_circuits(
         result.push_str("\n");
 
         let mut new_options = options.clone();
-        new_options.colorer = Some(PrintOptions::fixed_color(REMOVED_COLOR));
+        new_options.colorer = Some(PrintOptions::fixed_color(*REMOVED_COLOR));
         result.push_str(&indent(
             new_options.repr(old.clone())?,
             last_child_stack.len() * 2,
@@ -168,10 +169,10 @@ pub fn diff_circuits(
         write!(
             result,
             "{} {} {} {}\n",
-            color("Black: Same as original", None),
-            color("Blue: Children changed", SAME_SELF_COLOR),
-            color("Green: New", NEW_COLOR),
-            color("Red: Removed", REMOVED_COLOR)
+            color("Black: Same as original", CliColor::NONE),
+            color("Blue: Children changed", *SAME_SELF_COLOR),
+            color("Green: New", *NEW_COLOR),
+            color("Red: Removed", *REMOVED_COLOR)
         )
         .unwrap();
     }
