@@ -104,14 +104,19 @@ def gen_index_function(
         assert x.shape[:prefix_len] == index.shape
         suffix_len = x.ndim - prefix_len
         moved_x = x.movedim(check_canon_idx(index_dim, suffix_len) + prefix_len, 0)
-        flattened_x = moved_x.flatten(start_dim=1, end_dim=len(index.shape))  # end_dim is inclusive, so this is right
+        if moved_x.ndim == 1:
+            assert index.ndim == 0
+            return moved_x[index.long()]
+        else:
+            # end_dim is inclusive, so this is right
+            flattened_x = moved_x.flatten(start_dim=1, end_dim=len(index.shape))
         flattened_index = index.flatten().long()
         new_x = flattened_x[flattened_index, torch.arange(index.numel())]
         return new_x.reshape(moved_x.shape[1:])
 
     else:
         moved_out = torch.index_select(x, index_dim, index.long().flatten()).moveaxis(index_dim, 0)
-        return moved_out.reshape(*index.shape, *moved_out.shape[1:])
+        return moved_out.reshape((*index.shape, *moved_out.shape[1:]))
 
 
 assert_tensors_close = torch.testing.assert_close
@@ -151,7 +156,9 @@ einsum = lambda tensors_and_axes, out_axes: torch.einsum(*itertools.chain(*tenso
 def random_indices(input: torch.Tensor, shape, replacement: bool, seed: Optional[int] = None) -> torch.Tensor:
     shape = tuple(shape)
 
-    generator: Optional[torch.Generator] = torch.Generator().manual_seed(seed) if seed is not None else None
+    generator: Optional[torch.Generator] = (
+        torch.Generator(device=input.device).manual_seed(seed) if seed is not None else None
+    )
 
     return torch.multinomial(
         input,
@@ -200,3 +207,7 @@ def conv(dim, input, filter, stride, padding):
         f"{input_rearrange_string_o} -> {input_rearrange_string_i}",
         **{char_at(i): input.shape[i] for i in range(batch_rank)},
     )
+
+
+class OptimizingSymbolicSizeWarning(UserWarning):
+    ...
