@@ -974,7 +974,7 @@ macro_rules! lit_or {
     };
 }
 macro_rules! string_enum{
-    {$name:ident, ($($var:ident $(+$lit:literal)?),*)}=>{
+    {$name:ident, ($($var:ident $(=$lit:literal)?),*)}=>{
         #[derive(Copy, Clone, Debug, Hash, PartialEq, Eq, PartialOrd, Ord)]
         #[allow(non_camel_case_types)]
         pub enum $name{
@@ -1017,7 +1017,6 @@ macro_rules! string_enum{
         }
     }
 }
-
 string_enum! {
     TorchDtype,
     (
@@ -1037,23 +1036,23 @@ string_enum! {
 string_enum! {
     TorchDevice,
     (
-        Cpu+"cpu",
-        Cuda0+"cuda:0",
-        Cuda1+"cuda:1",
-        Cuda2+"cuda:2",
-        Cuda3+"cuda:3",
-        Cuda4+"cuda:4",
-        Cuda5+"cuda:5",
-        Cuda6+"cuda:6",
-        Cuda7+"cuda:7",
-        Cuda8+"cuda:8",
-        Cuda9+"cuda:9",
-        Cuda10+"cuda:10",
-        Cuda11+"cuda:11",
-        Cuda12+"cuda:12",
-        Cuda13+"cuda:13",
-        Cuda14+"cuda:14",
-        Cuda15+"cuda:15"
+        Cpu="cpu",
+        Cuda0="cuda:0",
+        Cuda1="cuda:1",
+        Cuda2="cuda:2",
+        Cuda3="cuda:3",
+        Cuda4="cuda:4",
+        Cuda5="cuda:5",
+        Cuda6="cuda:6",
+        Cuda7="cuda:7",
+        Cuda8="cuda:8",
+        Cuda9="cuda:9",
+        Cuda10="cuda:10",
+        Cuda11="cuda:11",
+        Cuda12="cuda:12",
+        Cuda13="cuda:13",
+        Cuda14="cuda:14",
+        Cuda15="cuda:15"
     )
 }
 
