@@ -65,6 +65,9 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     if !core::mem::size_of::<usize>() == 8 {
         return PyResult::Err(PyValueError::new_err("Only supports x64"));
     }
+    if !cfg!(target_endian = "little") {
+        return PyResult::Err(PyValueError::new_err("tried to build non little endian, rr_util::compact_data::TinyVecU8 relies on little endian"));
+    }
 
     m.add_class::<circuit_base::PyCircuitBase>()?;
 
